@@ -58,31 +58,6 @@ struct GSGrabPrivate
         GtkWidget *invisible;
 };
 
-static GdkCursor *
-get_cursor (void)
-{
-        GdkBitmap *empty_bitmap;
-        GdkCursor *cursor;
-        GdkColor   useless;
-        char       invisible_cursor_bits [] = { 0x0 };
-
-        useless.red = useless.green = useless.blue = 0;
-        useless.pixel = 0;
-
-        empty_bitmap = gdk_bitmap_create_from_data (NULL,
-                                                    invisible_cursor_bits,
-                                                    1, 1);
-
-        cursor = gdk_cursor_new_from_pixmap (empty_bitmap,
-                                             empty_bitmap,
-                                             &useless,
-                                             &useless, 0, 0);
-
-        g_object_unref (empty_bitmap);
-
-        return cursor;
-}
-
 static const char *
 grab_string (int status)
 {
@@ -118,7 +93,12 @@ static void
 xorg_lock_smasher_set_active (GSGrab  *grab,
                               gboolean active)
 {
-        int status;
+        int status, event, error;
+
+	if (!XF86MiscQueryExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &event, &error)) {
+		gs_debug ("No XFree86-Misc extension present");
+		return;
+	}
 
         if (active) {
                 gs_debug ("Enabling the x.org grab smasher");
@@ -128,22 +108,27 @@ xorg_lock_smasher_set_active (GSGrab  *grab,
 
         gdk_error_trap_push ();
 
-        status = XF86MiscSetGrabKeysState (GDK_DISPLAY (), active);
+        status = XF86MiscSetGrabKeysState (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), active);
 
         gdk_display_sync (gdk_display_get_default ());
-        gdk_error_trap_pop ();
+        error = gdk_error_trap_pop ();
 
         if (active && status == MiscExtGrabStateAlready) {
                 /* shut up, consider this success */
                 status = MiscExtGrabStateSuccess;
         }
 
-        gs_debug ("XF86MiscSetGrabKeysState(%s) returned %s\n",
-                  active ? "on" : "off",
-                  (status == MiscExtGrabStateSuccess ? "MiscExtGrabStateSuccess" :
-                   status == MiscExtGrabStateLocked  ? "MiscExtGrabStateLocked"  :
-                   status == MiscExtGrabStateAlready ? "MiscExtGrabStateAlready" :
-                   "unknown value"));
+        if (error == Success) {
+                gs_debug ("XF86MiscSetGrabKeysState(%s) returned %s\n",
+                          active ? "on" : "off",
+                          (status == MiscExtGrabStateSuccess ? "MiscExtGrabStateSuccess" :
+                           status == MiscExtGrabStateLocked  ? "MiscExtGrabStateLocked"  :
+                           status == MiscExtGrabStateAlready ? "MiscExtGrabStateAlready" :
+                           "unknown value"));
+        } else {
+                gs_debug ("XF86MiscSetGrabKeysState(%s) failed with error code %d\n",
+                          active ? "on" : "off", error);
+        }
 }
 #else
 static void
@@ -167,7 +152,15 @@ gs_grab_get_keyboard (GSGrab    *grab,
         status = gdk_keyboard_grab (window, FALSE, GDK_CURRENT_TIME);
 
         if (status == GDK_GRAB_SUCCESS) {
+                if (grab->priv->keyboard_grab_window != NULL) {
+                        g_object_remove_weak_pointer (G_OBJECT (grab->priv->keyboard_grab_window),
+                                                      (gpointer *) &grab->priv->keyboard_grab_window);
+                }
                 grab->priv->keyboard_grab_window = window;
+
+                g_object_add_weak_pointer (G_OBJECT (grab->priv->keyboard_grab_window),
+                                           (gpointer *) &grab->priv->keyboard_grab_window);
+
                 grab->priv->keyboard_grab_screen = screen;
         } else {
                 gs_debug ("Couldn't grab keyboard!  (%s)", grab_string (status));
@@ -188,7 +181,7 @@ gs_grab_get_mouse (GSGrab    *grab,
         g_return_val_if_fail (window != NULL, FALSE);
         g_return_val_if_fail (screen != NULL, FALSE);
 
-        cursor = get_cursor ();
+        cursor = gdk_cursor_new (GDK_BLANK_CURSOR);
 
         gs_debug ("Grabbing mouse widget=%X", (guint32) GDK_WINDOW_XID (window));
         status = gdk_pointer_grab (window, TRUE, 0, NULL,
@@ -196,7 +189,15 @@ gs_grab_get_mouse (GSGrab    *grab,
                                    GDK_CURRENT_TIME);
 
         if (status == GDK_GRAB_SUCCESS) {
+                if (grab->priv->mouse_grab_window != NULL) {
+                        g_object_remove_weak_pointer (G_OBJECT (grab->priv->mouse_grab_window),
+                                                      (gpointer *) &grab->priv->mouse_grab_window);
+                }
                 grab->priv->mouse_grab_window = window;
+
+                g_object_add_weak_pointer (G_OBJECT (grab->priv->mouse_grab_window),
+                                           (gpointer *) &grab->priv->mouse_grab_window);
+
                 grab->priv->mouse_grab_screen = screen;
                 grab->priv->mouse_hide_cursor = hide_cursor;
         }
@@ -209,6 +210,10 @@ gs_grab_get_mouse (GSGrab    *grab,
 void
 gs_grab_keyboard_reset (GSGrab *grab)
 {
+        if (grab->priv->keyboard_grab_window != NULL) {
+                g_object_remove_weak_pointer (G_OBJECT (grab->priv->keyboard_grab_window),
+                                              (gpointer *) &grab->priv->keyboard_grab_window);
+        }
         grab->priv->keyboard_grab_window = NULL;
         grab->priv->keyboard_grab_screen = NULL;
 }
@@ -227,6 +232,11 @@ gs_grab_release_keyboard (GSGrab *grab)
 void
 gs_grab_mouse_reset (GSGrab *grab)
 {
+        if (grab->priv->mouse_grab_window != NULL) {
+                g_object_remove_weak_pointer (G_OBJECT (grab->priv->mouse_grab_window),
+                                              (gpointer *) &grab->priv->mouse_grab_window);
+        }
+
         grab->priv->mouse_grab_window = NULL;
         grab->priv->mouse_grab_screen = NULL;
 }
@@ -374,12 +384,11 @@ gs_grab_nuke_focus (void)
 
         gdk_error_trap_push ();
 
-        XGetInputFocus (GDK_DISPLAY (), &focus, &rev);
+        XGetInputFocus (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &focus, &rev);
 
-        XSetInputFocus (GDK_DISPLAY (), None, RevertToNone, CurrentTime);
+        XSetInputFocus (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), None, RevertToNone, CurrentTime);
 
-        gdk_display_sync (gdk_display_get_default ());
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 }
 
 void
@@ -510,7 +519,7 @@ gs_grab_grab_offscreen (GSGrab *grab,
         gs_debug ("Grabbing an offscreen window");
 
         screen = gtk_invisible_get_screen (GTK_INVISIBLE (grab->priv->invisible));
-        res = gs_grab_grab_window (grab, grab->priv->invisible->window, screen, hide_cursor);
+        res = gs_grab_grab_window (grab, gtk_widget_get_window (grab->priv->invisible), screen, hide_cursor);
 
         return res;
 }
