@@ -35,7 +35,6 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
-#include <X11/XKBlib.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 
@@ -43,25 +42,23 @@
 #include <libgnomekbd/gkbd-indicator.h>
 #endif
 
-#ifdef WITH_LIBNOTIFY
-#include <libnotify/notify.h>
-#endif
-
 #include "gs-lock-plug.h"
 
 #include "gs-debug.h"
 
-#define KEY_LOCK_DIALOG_THEME "/apps/gnome-screensaver/lock_dialog_theme"
 #define GDM_FLEXISERVER_COMMAND "gdmflexiserver"
 #define GDM_FLEXISERVER_ARGS    "--startnew Standard"
 
-/* same as SMS ;) */
-#define NOTE_BUFFER_MAX_CHARS 160
-
 enum {
         AUTH_PAGE = 0,
 };
 
+enum {
+        LOCK_NONE = 0,
+        LOCK_CAPS = 1 << 0,
+        LOCK_NUM = 1 << 1,
+};
+
 #define FACE_ICON_SIZE 48
 #define DIALOG_TIMEOUT_MSEC 60000
 
@@ -71,6 +68,7 @@ static void gs_lock_plug_finalize   (GObject         *object);
 
 struct GSLockPlugPrivate
 {
+        GtkWidget   *frame;
         GtkWidget   *vbox;
         GtkWidget   *auth_action_area;
 
@@ -89,18 +87,11 @@ struct GSLockPlugPrivate
         GtkWidget   *auth_switch_button;
         GtkWidget   *auth_cancel_button;
         GtkWidget   *auth_logout_button;
-        GtkWidget   *auth_note_button;
-        GtkWidget   *note_tab;
-        GtkWidget   *note_tab_label;
-        GtkWidget   *note_text_view;
-        GtkWidget   *note_ok_button;
-        GtkWidget   *note_cancel_button;
 
         GtkWidget   *auth_prompt_kbd_layout_indicator;
 
-        gboolean     caps_lock_on;
+        int          kbd_lock_mode;
         gboolean     switch_enabled;
-        gboolean     leave_note_enabled;
         gboolean     logout_enabled;
         char        *logout_command;
         char        *status_message;
@@ -152,11 +143,7 @@ gs_lock_plug_style_set (GtkWidget *widget,
 
         plug = GS_LOCK_PLUG (widget);
 
-        if (! plug->priv->vbox) {
-                return;
-        }
-
-        gtk_container_set_border_width (GTK_CONTAINER (plug->priv->vbox), 12);
+        gtk_container_set_border_width (GTK_CONTAINER (plug->priv->vbox), 24);
         gtk_box_set_spacing (GTK_BOX (plug->priv->vbox), 12);
 
         gtk_container_set_border_width (GTK_CONTAINER (plug->priv->auth_action_area), 0);
@@ -283,37 +270,59 @@ dialog_timed_out (GSLockPlug *plug)
 
 
 static void
-capslock_update (GSLockPlug *plug,
-                 gboolean    is_on)
+kbd_lock_mode_update (GSLockPlug *plug,
+                      int         mode)
 {
+        if (plug->priv->kbd_lock_mode == mode) {
+                return;
+        }
 
-        plug->priv->caps_lock_on = is_on;
+        plug->priv->kbd_lock_mode = mode;
 
         if (plug->priv->auth_capslock_label == NULL) {
                 return;
         }
 
-        if (is_on) {
+        if ((mode & LOCK_CAPS) != 0 && (mode & LOCK_NUM) != 0) {
+                gtk_label_set_text (GTK_LABEL (plug->priv->auth_capslock_label),
+                                    _("You have the Caps & Num Lock keys on."));
+        } else if ((mode & LOCK_CAPS) != 0) {
                 gtk_label_set_text (GTK_LABEL (plug->priv->auth_capslock_label),
                                     _("You have the Caps Lock key on."));
+        } else if ((mode & LOCK_NUM) != 0) {
+                gtk_label_set_text (GTK_LABEL (plug->priv->auth_capslock_label),
+                                    _("You have the Num Lock key on."));
         } else {
                 gtk_label_set_text (GTK_LABEL (plug->priv->auth_capslock_label),
                                     "");
         }
 }
 
-static gboolean
-is_capslock_on (void)
+static int
+get_kbd_lock_mode (void)
 {
-        XkbStateRec states;
-        Display    *dsp;
+        GdkKeymap *keymap;
+        int        mode;
 
-        dsp = GDK_DISPLAY ();
-        if (XkbGetState (dsp, XkbUseCoreKbd, &states) != Success) {
-                return FALSE;
+        mode = LOCK_NONE;
+
+        keymap = gdk_keymap_get_default ();
+        if (keymap != NULL) {
+                gboolean res;
+
+                res = gdk_keymap_get_caps_lock_state (keymap);
+                if (res) {
+                        mode |= LOCK_CAPS;
+                }
+#if GTK_CHECK_VERSION(2,90,6)
+                res = gdk_keymap_get_num_lock_state (keymap);
+                if (res) {
+                        mode |= LOCK_NUM;
+                }
+#endif
         }
 
-        return (states.locked_mods & LockMask) != 0;
+        return mode;
 }
 
 static void
@@ -422,12 +431,12 @@ gs_lock_plug_run (GSLockPlug *plug)
 
         g_object_ref (plug);
 
-        was_modal = GTK_WINDOW (plug)->modal;
+        was_modal = gtk_window_get_modal (GTK_WINDOW (plug));
         if (!was_modal) {
                 gtk_window_set_modal (GTK_WINDOW (plug), TRUE);
         }
 
-        if (!GTK_WIDGET_VISIBLE (plug)) {
+        if (!gtk_widget_get_visible (GTK_WIDGET (plug))) {
                 gtk_widget_show (GTK_WIDGET (plug));
         }
 
@@ -543,55 +552,143 @@ rounded_rectangle (cairo_t *cr,
         cairo_close_path (cr);
 }
 
+/* copied from gdm-user.c */
+
+/**
+ * go_cairo_convert_data_to_pixbuf:
+ * @src: a pointer to pixel data in cairo format
+ * @dst: a pointer to pixel data in pixbuf format
+ * @width: image width
+ * @height: image height
+ * @rowstride: data rowstride
+ *
+ * Converts the pixel data stored in @src in CAIRO_FORMAT_ARGB32 cairo format
+ * to GDK_COLORSPACE_RGB pixbuf format and move them
+ * to @dst. If @src == @dst, pixel are converted in place.
+ **/
+
 static void
-image_set_from_pixbuf (GtkImage  *image,
-                       GdkPixbuf *source)
+go_cairo_convert_data_to_pixbuf (unsigned char *dst,
+                                 unsigned char const *src,
+                                 int width,
+                                 int height,
+                                 int rowstride)
 {
+        int i,j;
+        unsigned int t;
+        unsigned char a, b, c;
+
+        g_return_if_fail (dst != NULL);
+
+#define MULT(d,c,a,t) G_STMT_START { t = (a)? c * 255 / a: 0; d = t;} G_STMT_END
+
+        if (src == dst || src == NULL) {
+                for (i = 0; i < height; i++) {
+                        for (j = 0; j < width; j++) {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+                                MULT(a, dst[2], dst[3], t);
+                                MULT(b, dst[1], dst[3], t);
+                                MULT(c, dst[0], dst[3], t);
+                                dst[0] = a;
+                                dst[1] = b;
+                                dst[2] = c;
+#else
+                                MULT(a, dst[1], dst[0], t);
+                                MULT(b, dst[2], dst[0], t);
+                                MULT(c, dst[3], dst[0], t);
+                                dst[3] = dst[0];
+                                dst[0] = a;
+                                dst[1] = b;
+                                dst[2] = c;
+#endif
+                                dst += 4;
+                        }
+                        dst += rowstride - width * 4;
+                }
+        } else {
+                for (i = 0; i < height; i++) {
+                        for (j = 0; j < width; j++) {
+#if G_BYTE_ORDER == G_LITTLE_ENDIAN
+                                MULT(dst[0], src[2], src[3], t);
+                                MULT(dst[1], src[1], src[3], t);
+                                MULT(dst[2], src[0], src[3], t);
+                                dst[3] = src[3];
+#else
+                                MULT(dst[0], src[1], src[0], t);
+                                MULT(dst[1], src[2], src[0], t);
+                                MULT(dst[2], src[3], src[0], t);
+                                dst[3] = src[0];
+#endif
+                                src += 4;
+                                dst += 4;
+                        }
+                        src += rowstride - width * 4;
+                        dst += rowstride - width * 4;
+                }
+        }
+#undef MULT
+}
+
+static void
+cairo_to_pixbuf (guint8    *src_data,
+                 GdkPixbuf *dst_pixbuf)
+{
+        unsigned char *src;
+        unsigned char *dst;
+        guint          w;
+        guint          h;
+        guint          rowstride;
+
+        w = gdk_pixbuf_get_width (dst_pixbuf);
+        h = gdk_pixbuf_get_height (dst_pixbuf);
+        rowstride = gdk_pixbuf_get_rowstride (dst_pixbuf);
+
+        dst = gdk_pixbuf_get_pixels (dst_pixbuf);
+        src = src_data;
+
+        go_cairo_convert_data_to_pixbuf (dst, src, w, h, rowstride);
+}
+
+static GdkPixbuf *
+frame_pixbuf (GdkPixbuf *source)
+{
+        GdkPixbuf       *dest;
         cairo_t         *cr;
-        cairo_t         *cr_mask;
         cairo_surface_t *surface;
-        GdkPixmap       *pixmap;
-        GdkPixmap       *bitmask;
-        int              w;
-        int              h;
+        guint            w;
+        guint            h;
+        guint            rowstride;
         int              frame_width;
         double           radius;
-        GdkColor         color;
-        double           r;
-        double           g;
-        double           b;
+        guint8          *data;
 
         frame_width = 5;
 
         w = gdk_pixbuf_get_width (source) + frame_width * 2;
         h = gdk_pixbuf_get_height (source) + frame_width * 2;
-
         radius = w / 10;
 
-        pixmap = gdk_pixmap_new (GTK_WIDGET (image)->window, w, h, -1);
-        bitmask = gdk_pixmap_new (GTK_WIDGET (image)->window, w, h, 1);
+        dest = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+                               TRUE,
+                               8,
+                               w,
+                               h);
+        rowstride = gdk_pixbuf_get_rowstride (dest);
 
-        cr = gdk_cairo_create (pixmap);
-        cr_mask = gdk_cairo_create (bitmask);
 
-        /* setup mask */
-        cairo_rectangle (cr_mask, 0, 0, w, h);
-        cairo_set_operator (cr_mask, CAIRO_OPERATOR_CLEAR);
-        cairo_fill (cr_mask);
+        data = g_new0 (guint8, h * rowstride);
 
-        rounded_rectangle (cr_mask, 1.0, 0.5, 0.5, radius, w - 1, h - 1);
-        cairo_set_operator (cr_mask, CAIRO_OPERATOR_OVER);
-        cairo_set_source_rgb (cr_mask, 1, 1, 1);
-        cairo_fill (cr_mask);
-
-        color = GTK_WIDGET (image)->style->bg [GTK_STATE_NORMAL];
-        r = (float)color.red / 65535.0;
-        g = (float)color.green / 65535.0;
-        b = (float)color.blue / 65535.0;
+        surface = cairo_image_surface_create_for_data (data,
+                                                       CAIRO_FORMAT_ARGB32,
+                                                       w,
+                                                       h,
+                                                       rowstride);
+        cr = cairo_create (surface);
+        cairo_surface_destroy (surface);
 
         /* set up image */
         cairo_rectangle (cr, 0, 0, w, h);
-        cairo_set_source_rgb (cr, r, g, b);
+        cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0);
         cairo_fill (cr);
 
         rounded_rectangle (cr,
@@ -607,18 +704,28 @@ image_set_from_pixbuf (GtkImage  *image,
         surface = surface_from_pixbuf (source);
         cairo_set_source_surface (cr, surface, frame_width, frame_width);
         cairo_fill (cr);
-
-        gtk_image_set_from_pixmap (image, pixmap, bitmask);
-
         cairo_surface_destroy (surface);
 
-        g_object_unref (bitmask);
-        g_object_unref (pixmap);
+        cairo_to_pixbuf (data, dest);
 
-        cairo_destroy (cr_mask);
         cairo_destroy (cr);
+        g_free (data);
+
+        return dest;
 }
 
+/* end copied from gdm-user.c */
+
+static void
+image_set_from_pixbuf (GtkImage  *image,
+                       GdkPixbuf *source)
+{
+        GdkPixbuf *pixbuf;
+
+        pixbuf = frame_pixbuf (source);
+        gtk_image_set_from_pixbuf (image, pixbuf);
+        g_object_unref (pixbuf);
+}
 
 static gboolean
 check_user_file (const gchar *filename,
@@ -721,7 +828,7 @@ gs_lock_plug_show (GtkWidget *widget)
                 set_face_image (plug);
         }
 
-        capslock_update (plug, is_capslock_on ());
+        kbd_lock_mode_update (plug, get_kbd_lock_mode ());
 
         restart_cancel_timeout (plug);
 
@@ -767,7 +874,6 @@ static void
 gs_lock_plug_size_request (GtkWidget      *widget,
                            GtkRequisition *requisition)
 {
-        GSLockPlug *plug = GS_LOCK_PLUG (widget);
         int mod_width;
         int mod_height;
 
@@ -775,11 +881,6 @@ gs_lock_plug_size_request (GtkWidget      *widget,
                 GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->size_request (widget, requisition);
         }
 
-        /* don't constrain size when themed */
-        if (plug->priv->vbox == NULL) {
-                return;
-        }
-
         mod_width = requisition->height * 1.3;
         mod_height = requisition->width / 1.6;
         if (requisition->width < mod_width) {
@@ -961,7 +1062,7 @@ gs_lock_plug_close (GSLockPlug *plug)
         GdkEvent  *event;
 
         event = gdk_event_new (GDK_DELETE);
-        event->any.window = g_object_ref (widget->window);
+        event->any.window = g_object_ref (gtk_widget_get_window (widget));
         event->any.send_event = TRUE;
 
         gtk_main_do_event (event);
@@ -1035,7 +1136,7 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
 
         binding_set = gtk_binding_set_by_class (klass);
 
-        gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0,
+        gtk_binding_entry_add_signal (binding_set, GDK_KEY_Escape, 0,
                                       "close", 0);
 }
 
@@ -1052,74 +1153,6 @@ clear_clipboards (GSLockPlug *plug)
         gtk_clipboard_set_text (clipboard, "", -1);
 }
 
-static void
-take_note (GtkButton  *button,
-           GSLockPlug *plug)
-{
-        int page;
-
-        page = gtk_notebook_page_num (GTK_NOTEBOOK (plug->priv->notebook), plug->priv->note_tab);
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (plug->priv->notebook), page);
-        /* this counts as activity so restart the timer */
-        restart_cancel_timeout (plug);
-}
-
-static void
-submit_note (GtkButton  *button,
-             GSLockPlug *plug)
-{
-#ifdef WITH_LIBNOTIFY
-        char               *text;
-        char                summary[128];
-        char               *escaped_text;
-        GtkTextBuffer      *buffer;
-        GtkTextIter         start, end;
-        time_t              t;
-        struct tm          *tmp;
-        NotifyNotification *note;
-
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (plug->priv->notebook), AUTH_PAGE);
-        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (plug->priv->note_text_view));
-        gtk_text_buffer_get_bounds (buffer, &start, &end);
-        text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
-        gtk_text_buffer_set_text (buffer, "", 0);
-        escaped_text = g_markup_escape_text (text, -1);
-
-        t = time (NULL);
-        tmp = localtime (&t);
-        strftime (summary, 128, "%X", tmp);
-
-        notify_init ("gnome-screensaver-dialog");
-        note = notify_notification_new (summary, escaped_text, NULL, NULL);
-        notify_notification_set_timeout (note, NOTIFY_EXPIRES_NEVER);
-        notify_notification_show (note, NULL);
-        g_object_unref (note);
-
-        g_free (text);
-        g_free (escaped_text);
-
-        gs_lock_plug_response (plug, GS_LOCK_PLUG_RESPONSE_CANCEL);
-#endif /* WITH_LIBNOTIFY */
-}
-
-static void
-cancel_note (GtkButton  *button,
-             GSLockPlug *plug)
-{
-        GtkTextBuffer *buffer;
-
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (plug->priv->notebook), AUTH_PAGE);
-        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (plug->priv->note_text_view));
-        gtk_text_buffer_set_text (buffer, "", 0);
-
-        /* this counts as activity so restart the timer */
-        restart_cancel_timeout (plug);
-
-        gtk_window_set_default (GTK_WINDOW (plug), plug->priv->auth_unlock_button);
-
-        clear_clipboards (plug);
-}
-
 static void
 logout_button_clicked (GtkButton  *button,
                        GSLockPlug *plug)
@@ -1166,7 +1199,7 @@ gs_lock_plug_set_busy (GSLockPlug *plug)
         top_level = gtk_widget_get_toplevel (GTK_WIDGET (plug));
 
         cursor = gdk_cursor_new (GDK_WATCH);
-        gdk_window_set_cursor (top_level->window, cursor);
+        gdk_window_set_cursor (gtk_widget_get_window (top_level), cursor);
         gdk_cursor_unref (cursor);
 }
 
@@ -1179,7 +1212,7 @@ gs_lock_plug_set_ready (GSLockPlug *plug)
         top_level = gtk_widget_get_toplevel (GTK_WIDGET (plug));
 
         cursor = gdk_cursor_new (GDK_LEFT_PTR);
-        gdk_window_set_cursor (top_level->window, cursor);
+        gdk_window_set_cursor (gtk_widget_get_window (top_level), cursor);
         gdk_cursor_unref (cursor);
 }
 
@@ -1201,7 +1234,7 @@ gs_lock_plug_enable_prompt (GSLockPlug *plug,
         gtk_widget_set_sensitive (plug->priv->auth_prompt_entry, TRUE);
         gtk_widget_show (plug->priv->auth_prompt_entry);
 
-        if (! GTK_WIDGET_HAS_FOCUS (plug->priv->auth_prompt_entry)) {
+        if (!gtk_widget_has_focus (plug->priv->auth_prompt_entry)) {
                 gtk_widget_grab_focus (plug->priv->auth_prompt_entry);
         }
 
@@ -1253,21 +1286,15 @@ entry_key_press (GtkWidget   *widget,
                  GdkEventKey *event,
                  GSLockPlug  *plug)
 {
-        gboolean capslock_on;
-
         restart_cancel_timeout (plug);
 
-        capslock_on = is_capslock_on ();
-
-        if (capslock_on != plug->priv->caps_lock_on) {
-                capslock_update (plug, capslock_on);
-        }
+        kbd_lock_mode_update (plug, get_kbd_lock_mode ());
 
         /* if the input widget is visible and ready for input
          * then just carry on as usual
          */
-        if (GTK_WIDGET_VISIBLE (plug->priv->auth_prompt_entry) &&
-            GTK_WIDGET_IS_SENSITIVE (plug->priv->auth_prompt_entry)) {
+        if (gtk_widget_get_visible (plug->priv->auth_prompt_entry) &&
+            gtk_widget_get_sensitive (plug->priv->auth_prompt_entry)) {
                 return FALSE;
         }
 
@@ -1293,7 +1320,7 @@ gs_lock_plug_add_button (GSLockPlug  *plug,
 
         button = gtk_button_new_from_stock (button_text);
 
-        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+        gtk_widget_set_can_default (button, TRUE);
 
         gtk_widget_show (button);
 
@@ -1349,7 +1376,7 @@ create_page_one_buttons (GSLockPlug *plug)
 
         plug->priv->auth_switch_button =  gs_lock_plug_add_button (GS_LOCK_PLUG (plug),
                                                                    plug->priv->auth_action_area,
-                                                                   _("S_witch User..."));
+                                                                   _("S_witch User…"));
         gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (plug->priv->auth_action_area),
                                             plug->priv->auth_switch_button,
                                             TRUE);
@@ -1613,109 +1640,6 @@ switch_user_button_clicked (GtkButton  *button,
         do_user_switch (plug);
 }
 
-static char *
-get_dialog_theme_name (GSLockPlug *plug)
-{
-        char        *name;
-        GConfClient *client;
-
-        client = gconf_client_get_default ();
-        name = gconf_client_get_string (client, KEY_LOCK_DIALOG_THEME, NULL);
-        g_object_unref (client);
-
-        return name;
-}
-
-static gboolean
-load_theme (GSLockPlug *plug)
-{
-        char       *theme;
-        char       *filename;
-        char       *gtkbuilder;
-        char       *rc;
-        GtkBuilder *builder;
-        GtkWidget  *lock_dialog;
-        GError     *error=NULL;
-
-        theme = get_dialog_theme_name (plug);
-        if (theme == NULL) {
-                return FALSE;
-        }
-
-        filename = g_strdup_printf ("lock-dialog-%s.ui", theme);
-        gtkbuilder = g_build_filename (GTKBUILDERDIR, filename, NULL);
-        g_free (filename);
-        if (! g_file_test (gtkbuilder, G_FILE_TEST_IS_REGULAR)) {
-                g_free (gtkbuilder);
-                g_free (theme);
-                return FALSE;
-        }
-
-        filename = g_strdup_printf ("lock-dialog-%s.gtkrc", theme);
-        g_free (theme);
-
-        rc = g_build_filename (GTKBUILDERDIR, filename, NULL);
-        g_free (filename);
-        if (g_file_test (rc, G_FILE_TEST_IS_REGULAR)) {
-                gtk_rc_parse (rc);
-        }
-        g_free (rc);
-
-        builder = gtk_builder_new();
-
-        if (!gtk_builder_add_from_file (builder,gtkbuilder,&error)) {
-                g_warning ("Couldn't load builder file '%s': %s", gtkbuilder, error->message);
-                g_error_free(error);
-                g_free (gtkbuilder);
-                return FALSE;
-        }
-        g_free (gtkbuilder);
-
-        lock_dialog = GTK_WIDGET (gtk_builder_get_object(builder, "lock-dialog"));
-        gtk_container_add (GTK_CONTAINER (plug), lock_dialog);
-
-        plug->priv->vbox = NULL;
-        plug->priv->notebook = GTK_WIDGET (gtk_builder_get_object(builder, "notebook"));
-
-        plug->priv->auth_face_image = GTK_WIDGET (gtk_builder_get_object(builder, "auth-face-image"));
-        plug->priv->auth_action_area = GTK_WIDGET (gtk_builder_get_object(builder, "auth-action-area"));
-        plug->priv->auth_realname_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-realname-label"));
-        plug->priv->auth_username_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-username-label"));
-        plug->priv->auth_prompt_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-prompt-label"));
-        plug->priv->auth_prompt_entry = GTK_WIDGET (gtk_builder_get_object(builder, "auth-prompt-entry"));
-        plug->priv->auth_prompt_box = GTK_WIDGET (gtk_builder_get_object(builder, "auth-prompt-box"));
-        plug->priv->auth_capslock_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-capslock-label"));
-        plug->priv->auth_message_label = GTK_WIDGET (gtk_builder_get_object(builder, "auth-status-label"));
-        plug->priv->auth_unlock_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-unlock-button"));
-        plug->priv->auth_cancel_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-cancel-button"));
-        plug->priv->auth_logout_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-logout-button"));
-        plug->priv->auth_switch_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-switch-button"));
-        plug->priv->auth_note_button = GTK_WIDGET (gtk_builder_get_object(builder, "auth-note-button"));
-        plug->priv->note_tab = GTK_WIDGET (gtk_builder_get_object(builder, "note-tab"));
-        plug->priv->note_tab_label = GTK_WIDGET (gtk_builder_get_object(builder, "note-tab-label"));
-        plug->priv->note_ok_button = GTK_WIDGET (gtk_builder_get_object(builder, "note-ok-button"));
-        plug->priv->note_text_view = GTK_WIDGET (gtk_builder_get_object(builder, "note-text-view"));
-        plug->priv->note_cancel_button = GTK_WIDGET (gtk_builder_get_object(builder, "note-cancel-button"));
-
-        /* Placeholder for the keyboard indicator */
-        plug->priv->auth_prompt_kbd_layout_indicator = GTK_WIDGET (gtk_builder_get_object(builder, "auth-prompt-kbd-layout-indicator"));
-        if (plug->priv->auth_logout_button != NULL) {
-                gtk_widget_set_no_show_all (plug->priv->auth_logout_button, TRUE);
-        }
-        if (plug->priv->auth_switch_button != NULL) {
-                gtk_widget_set_no_show_all (plug->priv->auth_switch_button, TRUE);
-        }
-        if (plug->priv->auth_note_button != NULL) {
-                gtk_widget_set_no_show_all (plug->priv->auth_note_button, TRUE);
-        }
-
-        gtk_widget_show_all (lock_dialog);
-
-        plug->priv->status_message_label = GTK_WIDGET (gtk_builder_get_object(builder, "status-message-label"));
-
-        return TRUE;
-}
-
 static int
 delete_handler (GSLockPlug  *plug,
                 GdkEventAny *event,
@@ -1726,18 +1650,6 @@ delete_handler (GSLockPlug  *plug,
         return TRUE; /* Do not destroy */
 }
 
-static void
-on_note_text_buffer_changed (GtkTextBuffer *buffer,
-                             GSLockPlug    *plug)
-{
-        int len;
-
-        len = gtk_text_buffer_get_char_count (buffer);
-        if (len > NOTE_BUFFER_MAX_CHARS) {
-                gtk_widget_set_sensitive (plug->priv->note_text_view, FALSE);
-        }
-}
-
 static void
 gs_lock_plug_init (GSLockPlug *plug)
 {
@@ -1747,45 +1659,31 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         clear_clipboards (plug);
 
-#ifdef WITH_LIBNOTIFY
-        plug->priv->leave_note_enabled = TRUE;
-#else
-        plug->priv->leave_note_enabled = FALSE;
-#endif
+        plug->priv->frame = gtk_frame_new (NULL);
+        gtk_frame_set_shadow_type (GTK_FRAME (plug->priv->frame), GTK_SHADOW_OUT);
+        gtk_container_add (GTK_CONTAINER (plug), plug->priv->frame);
 
-        if (! load_theme (plug)) {
-                gs_debug ("Unable to load theme!");
+        plug->priv->vbox = gtk_vbox_new (FALSE, 0);
+        gtk_container_add (GTK_CONTAINER (plug->priv->frame), plug->priv->vbox);
 
-                plug->priv->vbox = gtk_vbox_new (FALSE, 0);
+        /* Notebook */
+        plug->priv->notebook = gtk_notebook_new ();
+        gtk_notebook_set_show_tabs (GTK_NOTEBOOK (plug->priv->notebook), FALSE);
+        gtk_notebook_set_show_border (GTK_NOTEBOOK (plug->priv->notebook), FALSE);
+        gtk_box_pack_start (GTK_BOX (plug->priv->vbox), plug->priv->notebook, TRUE, TRUE, 0);
 
-                gtk_container_add (GTK_CONTAINER (plug), plug->priv->vbox);
+        /* Page 1 */
 
-                /* Notebook */
+        create_page_one (plug);
 
-                plug->priv->notebook = gtk_notebook_new ();
-                gtk_notebook_set_show_tabs (GTK_NOTEBOOK (plug->priv->notebook), FALSE);
-                gtk_notebook_set_show_border (GTK_NOTEBOOK (plug->priv->notebook), FALSE);
-                gtk_box_pack_start (GTK_BOX (plug->priv->vbox), plug->priv->notebook, TRUE, TRUE, 0);
-
-                /* Page 1 */
-
-                create_page_one (plug);
-
-                gtk_widget_show_all (plug->priv->vbox);
-        }
-
-        if (plug->priv->note_text_view != NULL) {
-                GtkTextBuffer *buffer;
-                buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (plug->priv->note_text_view));
-                g_signal_connect (buffer, "changed", G_CALLBACK (on_note_text_buffer_changed), plug);
-        }
+        gtk_widget_show_all (plug->priv->frame);
 
         /* Layout indicator */
 #ifdef WITH_KBD_LAYOUT_INDICATOR
         if (plug->priv->auth_prompt_kbd_layout_indicator != NULL) {
                 XklEngine *engine;
 
-                engine = xkl_engine_get_instance (GDK_DISPLAY ());
+                engine = xkl_engine_get_instance (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
                 if (xkl_engine_get_num_groups (engine) > 1) {
                         GtkWidget *layout_indicator;
 
@@ -1807,13 +1705,6 @@ gs_lock_plug_init (GSLockPlug *plug)
         }
 #endif
 
-        if (plug->priv->auth_note_button != NULL) {
-                if (plug->priv->leave_note_enabled) {
-                        gtk_widget_show_all (plug->priv->auth_note_button);
-                } else {
-                        gtk_widget_hide (plug->priv->auth_note_button);
-                }
-        }
         if (plug->priv->auth_switch_button != NULL) {
                 if (plug->priv->switch_enabled) {
                         gtk_widget_show_all (plug->priv->auth_switch_button);
@@ -1870,19 +1761,6 @@ gs_lock_plug_init (GSLockPlug *plug)
                                   G_CALLBACK (switch_user_button_clicked), plug);
         }
 
-        if (plug->priv->auth_note_button != NULL) {
-                g_signal_connect (plug->priv->auth_note_button, "clicked",
-                                  G_CALLBACK (take_note), plug);
-                g_signal_connect (plug->priv->note_ok_button, "clicked",
-                                  G_CALLBACK (submit_note), plug);
-                g_signal_connect (plug->priv->note_cancel_button, "clicked",
-                                  G_CALLBACK (cancel_note), plug);
-        }
-
-        if (plug->priv->note_tab_label != NULL) {
-                expand_string_for_label (plug->priv->note_tab_label);
-        }
-
         if (plug->priv->auth_logout_button != NULL) {
                 g_signal_connect (plug->priv->auth_logout_button, "clicked",
                                   G_CALLBACK (logout_button_clicked), plug);
