@@ -64,6 +64,7 @@ struct GSWindowPrivate
 
         GdkRectangle geometry;
         guint      obscured : 1;
+        guint      dialog_up : 1;
 
         guint      lock_enabled : 1;
         guint      user_switch_enabled : 1;
@@ -83,7 +84,7 @@ struct GSWindowPrivate
         GtkWidget *info_bar;
         GtkWidget *info_content;
 
-        GdkPixmap *background_pixmap;
+        cairo_surface_t *background_surface;
 
         guint      popup_dialog_idle_id;
 
@@ -118,14 +119,13 @@ struct GSWindowPrivate
 enum {
         ACTIVITY,
         DEACTIVATED,
-        DIALOG_UP,
-        DIALOG_DOWN,
         LAST_SIGNAL
 };
 
 enum {
         PROP_0,
         PROP_OBSCURED,
+        PROP_DIALOG_UP,
         PROP_LOCK_ENABLED,
         PROP_LOGOUT_ENABLED,
         PROP_KEYBOARD_ENABLED,
@@ -144,25 +144,10 @@ static void
 set_invisible_cursor (GdkWindow *window,
                       gboolean   invisible)
 {
-        GdkBitmap *empty_bitmap;
         GdkCursor *cursor = NULL;
-        GdkColor   useless;
-        char       invisible_cursor_bits [] = { 0x0 };
 
         if (invisible) {
-                useless.red = useless.green = useless.blue = 0;
-                useless.pixel = 0;
-
-                empty_bitmap = gdk_bitmap_create_from_data (window,
-                                                            invisible_cursor_bits,
-                                                            1, 1);
-
-                cursor = gdk_cursor_new_from_pixmap (empty_bitmap,
-                                                     empty_bitmap,
-                                                     &useless,
-                                                     &useless, 0, 0);
-
-                g_object_unref (empty_bitmap);
+                cursor = gdk_cursor_new (GDK_BLANK_CURSOR);
         }
 
         gdk_window_set_cursor (window, cursor);
@@ -189,10 +174,10 @@ gs_window_override_user_time (GSWindow *window)
                  * NOTE: Last resort for D-BUS or other non-interactive
                  *       openings.  Causes roundtrip to server.  Lame.
                  */
-                ev_time = gdk_x11_get_server_time (GTK_WIDGET (window)->window);
+                ev_time = gdk_x11_get_server_time (gtk_widget_get_window (GTK_WIDGET (window)));
         }
 
-        gdk_x11_window_set_user_time (GTK_WIDGET (window)->window, ev_time);
+        gdk_x11_window_set_user_time (gtk_widget_get_window (GTK_WIDGET (window)), ev_time);
 }
 
 static void
@@ -231,7 +216,7 @@ clear_children (Window window)
         int               status;
 
         children = NULL;
-        status = XQueryTree (GDK_DISPLAY (), window, &root, &parent, &children, &n_children);
+        status = XQueryTree (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), window, &root, &parent, &children, &n_children);
 
         if (status == 0) {
                 if (children) {
@@ -246,7 +231,7 @@ clear_children (Window window)
 
                         child = children [--n_children];
 
-                        XClearWindow (GDK_DISPLAY (), child);
+                        XClearWindow (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), child);
                         clear_children (child);
                 }
 
@@ -263,124 +248,65 @@ widget_clear_all_children (GtkWidget *widget)
 
         gdk_error_trap_push ();
 
-        w = widget->window;
+        w = gtk_widget_get_window (widget);
 
         clear_children (GDK_WINDOW_XID (w));
 
-        gdk_display_sync (gtk_widget_get_display (widget));
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
+}
+
+static void
+gs_window_reset_background_surface (GSWindow *window)
+{
+        cairo_pattern_t *pattern;
+        pattern = cairo_pattern_create_for_surface (window->priv->background_surface);
+        gdk_window_set_background_pattern (gtk_widget_get_window (GTK_WIDGET (window)),
+                                           pattern);
+        cairo_pattern_destroy (pattern);
+        gtk_widget_queue_draw (GTK_WIDGET (window));
 }
 
 void
-gs_window_set_background_pixmap (GSWindow  *window,
-                                 GdkPixmap *pixmap)
+gs_window_set_background_surface (GSWindow        *window,
+                                  cairo_surface_t *surface)
 {
         g_return_if_fail (GS_IS_WINDOW (window));
 
-        if (window->priv->background_pixmap != NULL) {
-                g_object_unref (window->priv->background_pixmap);
+        if (window->priv->background_surface != NULL) {
+                cairo_surface_destroy (window->priv->background_surface);
         }
 
-        if (pixmap != NULL) {
-                window->priv->background_pixmap = g_object_ref (pixmap);
-                gdk_window_set_back_pixmap (GTK_WIDGET (window)->window,
-                                            pixmap,
-                                            FALSE);
+        if (surface != NULL) {
+                window->priv->background_surface = cairo_surface_reference (surface);
+                gs_window_reset_background_surface (window);
         }
 }
 
 static void
-gs_window_clear_to_background_pixmap (GSWindow *window)
+gs_window_clear_to_background_surface (GSWindow *window)
 {
-        GtkStateType state;
-        GtkStyle    *style;
-
         g_return_if_fail (GS_IS_WINDOW (window));
 
-        if (! GTK_WIDGET_VISIBLE (GTK_WIDGET (window))) {
+        if (!gtk_widget_get_visible (GTK_WIDGET (window))) {
                 return;
         }
 
-        if (window->priv->background_pixmap == NULL) {
-                /* don't allow null pixmaps */
+        if (window->priv->background_surface == NULL) {
                 return;
         }
 
         gs_debug ("Clearing window to background pixmap");
-
-        style = gtk_style_copy (GTK_WIDGET (window)->style);
-
-        state = (GtkStateType) 0;
-        while (state < (GtkStateType) G_N_ELEMENTS (GTK_WIDGET (window)->style->bg_pixmap)) {
-
-                if (style->bg_pixmap[state] != NULL) {
-                        g_object_unref (style->bg_pixmap[state]);
-                }
-
-                style->bg_pixmap[state] = g_object_ref (window->priv->background_pixmap);
-                state++;
-        }
-
-        gtk_widget_set_style (GTK_WIDGET (window), style);
-        g_object_unref (style);
-
-        if (window->priv->background_pixmap != NULL) {
-                gdk_window_set_back_pixmap (GTK_WIDGET (window)->window,
-                                            window->priv->background_pixmap,
-                                            FALSE);
-        }
-
-        gdk_window_clear (GTK_WIDGET (window)->window);
-
-        gdk_flush ();
+        gs_window_reset_background_surface (window);
 }
 
 static void
 clear_widget (GtkWidget *widget)
 {
-        GdkColor     color = { 0, 0x0000, 0x0000, 0x0000 };
-        GdkColormap *colormap;
-        GtkStateType state;
-        GtkStyle    *style;
+        GdkColor color = { 0, 0x0000, 0x0000, 0x0000 };
 
-        if (! GTK_WIDGET_VISIBLE (widget)) {
-                return;
-        }
-
-        gs_debug ("Clearing widget");
-
-        state = (GtkStateType) 0;
-        while (state < (GtkStateType) G_N_ELEMENTS (widget->style->bg)) {
-                gtk_widget_modify_bg (widget, state, &color);
-                state++;
-        }
-
-        style = gtk_style_copy (widget->style);
-
-        state = (GtkStateType) 0;
-        while (state < (GtkStateType) G_N_ELEMENTS (widget->style->bg_pixmap)) {
-
-                if (style->bg_pixmap[state] != NULL) {
-                        g_object_unref (style->bg_pixmap[state]);
-                }
-
-                style->bg_pixmap[state] = NULL;
-                state++;
-        }
-
-        colormap = gdk_drawable_get_colormap (widget->window);
-        gdk_colormap_alloc_color (colormap, &color, FALSE, TRUE);
-        gdk_window_set_background (widget->window, &color);
-
-        gtk_widget_set_style (widget, style);
-        g_object_unref (style);
-
-        gdk_window_clear (widget->window);
-
-        /* If a screensaver theme adds child windows we need to clear them too */
-        widget_clear_all_children (widget);
-
-        gdk_flush ();
+        gdk_window_set_background (gtk_widget_get_window (widget),
+                                   &color);
+        gtk_widget_queue_draw (GTK_WIDGET (widget));
 }
 
 void
@@ -392,19 +318,24 @@ gs_window_clear (GSWindow *window)
         clear_widget (window->priv->drawing_area);
 }
 
-static GdkRegion *
+static cairo_region_t *
 get_outside_region (GSWindow *window)
 {
-        int        i;
-        GdkRegion *region;
+        int             i;
+        cairo_region_t *region;
 
-        region = gdk_region_new ();
+        region = cairo_region_create ();
         for (i = 0; i < window->priv->monitor; i++) {
                 GdkRectangle geometry;
+                cairo_rectangle_int_t rectangle;
 
-                gdk_screen_get_monitor_geometry (GTK_WINDOW (window)->screen,
+                gdk_screen_get_monitor_geometry (gtk_window_get_screen (GTK_WINDOW (window)),
                                                    i, &geometry);
-                gdk_region_union_with_rect (region, &geometry);
+                rectangle.x = geometry.x;
+                rectangle.y = geometry.y;
+                rectangle.width = geometry.width;
+                rectangle.height = geometry.height;
+                cairo_region_union_rectangle (region, &rectangle);
         }
 
         return region;
@@ -413,13 +344,13 @@ get_outside_region (GSWindow *window)
 static void
 update_geometry (GSWindow *window)
 {
-        GdkRectangle geometry;
-        GdkRegion   *outside_region;
-        GdkRegion   *monitor_region;
+        GdkRectangle    geometry;
+        cairo_region_t *outside_region;
+        cairo_region_t *monitor_region;
 
         outside_region = get_outside_region (window);
 
-        gdk_screen_get_monitor_geometry (GTK_WINDOW (window)->screen,
+        gdk_screen_get_monitor_geometry (gtk_window_get_screen (GTK_WINDOW (window)),
                                          window->priv->monitor,
                                          &geometry);
         gs_debug ("got geometry for monitor %d: x=%d y=%d w=%d h=%d",
@@ -428,12 +359,12 @@ update_geometry (GSWindow *window)
                   geometry.y,
                   geometry.width,
                   geometry.height);
-        monitor_region = gdk_region_rectangle (&geometry);
-        gdk_region_subtract (monitor_region, outside_region);
-        gdk_region_destroy (outside_region);
+        monitor_region = cairo_region_create_rectangle ((const cairo_rectangle_int_t *)&geometry);
+        cairo_region_subtract (monitor_region, outside_region);
+        cairo_region_destroy (outside_region);
 
-        gdk_region_get_clipbox (monitor_region, &geometry);
-        gdk_region_destroy (monitor_region);
+        cairo_region_get_extents (monitor_region, (cairo_rectangle_int_t *)&geometry);
+        cairo_region_destroy (monitor_region);
 
         gs_debug ("using geometry for monitor %d: x=%d y=%d w=%d h=%d",
                   window->priv->monitor,
@@ -466,7 +397,7 @@ gs_window_move_resize_window (GSWindow *window,
 
         widget = GTK_WIDGET (window);
 
-        g_assert (GTK_WIDGET_REALIZED (widget));
+        g_assert (gtk_widget_get_realized (widget));
 
         gs_debug ("Move and/or resize window on monitor %d: x=%d y=%d w=%d h=%d",
                   window->priv->monitor,
@@ -476,17 +407,17 @@ gs_window_move_resize_window (GSWindow *window,
                   window->priv->geometry.height);
 
         if (move && resize) {
-                gdk_window_move_resize (widget->window,
+                gdk_window_move_resize (gtk_widget_get_window (widget),
                                         window->priv->geometry.x,
                                         window->priv->geometry.y,
                                         window->priv->geometry.width,
                                         window->priv->geometry.height);
         } else if (move) {
-                gdk_window_move (widget->window,
+                gdk_window_move (gtk_widget_get_window (widget),
                                  window->priv->geometry.x,
                                  window->priv->geometry.y);
         } else if (resize) {
-                gdk_window_resize (widget->window,
+                gdk_window_resize (gtk_widget_get_window (widget),
                                    window->priv->geometry.width,
                                    window->priv->geometry.height);
         }
@@ -626,45 +557,27 @@ get_best_visual_for_screen (GdkScreen *screen)
         g_free (std_output);
         g_free (command);
 
-        return visual;
-}
-
-static GdkColormap *
-get_best_colormap_for_screen (GdkScreen *screen)
-{
-        GdkColormap *colormap;
-        GdkVisual   *visual;
-
-        g_return_val_if_fail (screen != NULL, NULL);
-
-        visual = get_best_visual_for_screen (screen);
-
-        colormap = NULL;
-        if (visual != NULL) {
-                colormap = gdk_colormap_new (visual, FALSE);
-        }
-
-        return colormap;
+        return g_object_ref (visual);
 }
 
 static void
-widget_set_best_colormap (GtkWidget *widget)
+widget_set_best_visual (GtkWidget *widget)
 {
-        GdkColormap *colormap;
+        GdkVisual *visual;
 
         g_return_if_fail (widget != NULL);
 
-        colormap = get_best_colormap_for_screen (gtk_widget_get_screen (widget));
-        if (colormap != NULL) {
-                gtk_widget_set_colormap (widget, colormap);
-                g_object_unref (colormap);
+        visual = get_best_visual_for_screen (gtk_widget_get_screen (widget));
+        if (visual != NULL) {
+                gtk_widget_set_visual (widget, visual);
+                g_object_unref (visual);
         }
 }
 
 static void
 gs_window_real_realize (GtkWidget *widget)
 {
-        widget_set_best_colormap (widget);
+        widget_set_best_visual (widget);
 
         if (GTK_WIDGET_CLASS (gs_window_parent_class)->realize) {
                 GTK_WIDGET_CLASS (gs_window_parent_class)->realize (widget);
@@ -687,7 +600,7 @@ watchdog_timer (GSWindow *window)
 {
         GtkWidget *widget = GTK_WIDGET (window);
 
-        gdk_window_focus (widget->window, GDK_CURRENT_TIME);
+        gdk_window_focus (gtk_widget_get_window (widget), GDK_CURRENT_TIME);
 
         return TRUE;
 }
@@ -748,7 +661,7 @@ gs_window_raise (GSWindow *window)
 
         gs_debug ("Raising screensaver window");
 
-        win = GTK_WIDGET (window)->window;
+        win = gtk_widget_get_window (GTK_WIDGET (window));
 
         gdk_window_raise (win);
 }
@@ -773,7 +686,7 @@ x11_window_is_ours (Window window)
 static void
 unshape_window (GSWindow *window)
 {
-        gdk_window_shape_combine_region (GTK_WIDGET (window)->window,
+        gdk_window_shape_combine_region (gtk_widget_get_window (GTK_WIDGET (window)),
                                          NULL,
                                          0,
                                          0);
@@ -849,13 +762,12 @@ select_popup_events (void)
         gdk_error_trap_push ();
 
         memset (&attr, 0, sizeof (attr));
-        XGetWindowAttributes (GDK_DISPLAY (), GDK_ROOT_WINDOW (), &attr);
+        XGetWindowAttributes (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), GDK_ROOT_WINDOW (), &attr);
 
         events = SubstructureNotifyMask | attr.your_event_mask;
-        XSelectInput (GDK_DISPLAY (), GDK_ROOT_WINDOW (), events);
+        XSelectInput (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), GDK_ROOT_WINDOW (), events);
 
-        gdk_display_sync (gdk_display_get_default ());
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 }
 
 static void
@@ -867,13 +779,12 @@ window_select_shape_events (GSWindow *window)
 
         gdk_error_trap_push ();
 
-        if (XShapeQueryExtension (GDK_DISPLAY (), &window->priv->shape_event_base, &shape_error_base)) {
+        if (XShapeQueryExtension (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &window->priv->shape_event_base, &shape_error_base)) {
                 events = ShapeNotifyMask;
-                XShapeSelectInput (GDK_DISPLAY (), GDK_WINDOW_XID (GTK_WIDGET (window)->window), events);
+                XShapeSelectInput (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), GDK_WINDOW_XID (gtk_widget_get_window (GTK_WIDGET (window))), events);
         }
 
-        gdk_display_sync (gdk_display_get_default ());
-        gdk_error_trap_pop ();
+        gdk_error_trap_pop_ignored ();
 #endif
 }
 
@@ -888,7 +799,7 @@ gs_window_real_show (GtkWidget *widget)
 
         gs_window_clear (GS_WINDOW (widget));
 
-        set_invisible_cursor (widget->window, TRUE);
+        set_invisible_cursor (gtk_widget_get_window (widget), TRUE);
 
         window = GS_WINDOW (widget);
         if (window->priv->timer) {
@@ -1023,20 +934,7 @@ gs_window_destroy (GSWindow *window)
 {
         g_return_if_fail (GS_IS_WINDOW (window));
 
-        if (window->priv->lock_pid > 0) {
-                gs_window_dialog_finish (window);
-        }
-
-        remove_popup_dialog_idle (window);
-        remove_command_watches (window);
-        remove_watchdog_timer (window);
-
-        if (window->priv->lock_box != NULL) {
-                gtk_container_remove (GTK_CONTAINER (window->priv->vbox), GTK_WIDGET (window->priv->lock_box));
-                window->priv->lock_box = NULL;
-
-                g_signal_emit (window, signals [DIALOG_DOWN], 0);
-        }
+        gs_window_cancel_unlock_request (window);
 
         gtk_widget_destroy (GTK_WIDGET (window));
 }
@@ -1046,7 +944,7 @@ gs_window_get_gdk_window (GSWindow *window)
 {
         g_return_val_if_fail (GS_IS_WINDOW (window), NULL);
 
-        return GTK_WIDGET (window)->window;
+        return gtk_widget_get_window (GTK_WIDGET (window));
 }
 
 GtkWidget *
@@ -1127,7 +1025,7 @@ spawn_on_window (GSWindow *window,
         }
 
         error = NULL;
-        result = gdk_spawn_on_screen_with_pipes (GTK_WINDOW (window)->screen,
+        result = gdk_spawn_on_screen_with_pipes (gtk_window_get_screen (GTK_WINDOW (window)),
                                                  NULL,
                                                  argv,
                                                  NULL,
@@ -1545,6 +1443,43 @@ shake_dialog (GSWindow *window)
         maybe_kill_dialog (window);
 }
 
+static void
+window_set_dialog_up (GSWindow *window,
+                      gboolean  dialog_up)
+{
+        if (window->priv->dialog_up == dialog_up) {
+                return;
+        }
+
+        window->priv->dialog_up = dialog_up;
+        g_object_notify (G_OBJECT (window), "dialog-up");
+}
+
+static void
+popdown_dialog (GSWindow *window)
+{
+        gs_window_dialog_finish (window);
+
+        gtk_widget_show (window->priv->drawing_area);
+
+        gs_window_clear (window);
+        set_invisible_cursor (gtk_widget_get_window (GTK_WIDGET (window)), TRUE);
+
+        window_set_dialog_up (window, FALSE);
+
+        /* reset the pointer positions */
+        window->priv->last_x = -1;
+        window->priv->last_y = -1;
+
+        if (window->priv->lock_box != NULL) {
+                gtk_container_remove (GTK_CONTAINER (window->priv->vbox), GTK_WIDGET (window->priv->lock_box));
+                window->priv->lock_box = NULL;
+        }
+
+        remove_popup_dialog_idle (window);
+        remove_command_watches (window);
+}
+
 static gboolean
 lock_command_watch (GIOChannel   *source,
                     GIOCondition  condition,
@@ -1610,22 +1545,12 @@ lock_command_watch (GIOChannel   *source,
         }
 
         if (finished) {
-                gs_window_dialog_finish (window);
+                popdown_dialog (window);
 
                 if (window->priv->dialog_response == DIALOG_RESPONSE_OK) {
                         add_emit_deactivated_idle (window);
                 }
 
-                gtk_widget_show (window->priv->drawing_area);
-
-                gs_window_clear (window);
-                set_invisible_cursor (GTK_WIDGET (window)->window, TRUE);
-                g_signal_emit (window, signals [DIALOG_DOWN], 0);
-
-                /* reset the pointer positions */
-                window->priv->last_x = -1;
-                window->priv->last_y = -1;
-
                 window->priv->lock_watch_id = 0;
 
                 return FALSE;
@@ -1662,8 +1587,8 @@ is_user_switch_enabled (GSWindow *window)
         return window->priv->user_switch_enabled;
 }
 
-static gboolean
-popup_dialog_idle (GSWindow *window)
+static void
+popup_dialog (GSWindow *window)
 {
         gboolean  result;
         char     *tmp;
@@ -1698,9 +1623,9 @@ popup_dialog_idle (GSWindow *window)
 
         gtk_widget_hide (window->priv->drawing_area);
 
-        gs_window_clear_to_background_pixmap (window);
+        gs_window_clear_to_background_surface (window);
 
-        set_invisible_cursor (GTK_WIDGET (window)->window, FALSE);
+        set_invisible_cursor (gtk_widget_get_window (GTK_WIDGET (window)), FALSE);
 
         window->priv->dialog_quit_requested = FALSE;
         window->priv->dialog_shake_in_progress = FALSE;
@@ -1716,6 +1641,12 @@ popup_dialog_idle (GSWindow *window)
         }
 
         g_string_free (command, TRUE);
+}
+
+static gboolean
+popup_dialog_idle (GSWindow *window)
+{
+        popup_dialog (window);
 
         window->priv->popup_dialog_idle_id = 0;
 
@@ -1729,7 +1660,7 @@ gs_window_request_unlock (GSWindow *window)
 
         gs_debug ("Requesting unlock");
 
-        if (! GTK_WIDGET_VISIBLE (GTK_WIDGET (window))) {
+        if (! gtk_widget_get_visible (GTK_WIDGET (window))) {
                 gs_debug ("Request unlock but window is not visible!");
                 return;
         }
@@ -1748,7 +1679,7 @@ gs_window_request_unlock (GSWindow *window)
                 add_popup_dialog_idle (window);
         }
 
-        g_signal_emit (window, signals [DIALOG_UP], 0);
+        window_set_dialog_up (window, TRUE);
 }
 
 void
@@ -1760,13 +1691,7 @@ gs_window_cancel_unlock_request (GSWindow  *window)
 	 */
         g_return_if_fail (GS_IS_WINDOW (window));
 
-        if (window->priv->lock_socket == NULL) {
-                return;
-        }
-
-        if (window->priv->lock_pid > 0) {
-                kill (window->priv->lock_pid, SIGTERM);
-        }
+        popdown_dialog (window);
 }
 
 void
@@ -1799,7 +1724,7 @@ gs_window_get_screen (GSWindow  *window)
 {
         g_return_val_if_fail (GS_IS_WINDOW (window), NULL);
 
-        return GTK_WINDOW (window)->screen;
+        return gtk_window_get_screen (GTK_WINDOW (window));
 }
 
 void
@@ -1986,6 +1911,9 @@ gs_window_get_property (GObject    *object,
         case PROP_OBSCURED:
                 g_value_set_boolean (value, self->priv->obscured);
                 break;
+        case PROP_DIALOG_UP:
+                g_value_set_boolean (value, self->priv->dialog_up);
+                break;
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
                 break;
@@ -1998,10 +1926,10 @@ queue_key_event (GSWindow    *window,
 {
         /* Eat the first return, enter, escape, or space */
         if (window->priv->key_events == NULL
-            && (event->keyval == GDK_Return
-                || event->keyval == GDK_KP_Enter
-                || event->keyval == GDK_Escape
-                || event->keyval == GDK_space)) {
+            && (event->keyval == GDK_KEY_Return
+                || event->keyval == GDK_KEY_KP_Enter
+                || event->keyval == GDK_KEY_Escape
+                || event->keyval == GDK_KEY_space)) {
                 return;
         }
 
@@ -2009,9 +1937,9 @@ queue_key_event (GSWindow    *window,
            something is wrong */
         /* Don't queue keys that may cause focus navigation in the dialog */
         if (g_list_length (window->priv->key_events) < MAX_QUEUED_EVENTS
-            && event->keyval != GDK_Tab
-            && event->keyval != GDK_Up
-            && event->keyval != GDK_Down) {
+            && event->keyval != GDK_KEY_Tab
+            && event->keyval != GDK_KEY_Up
+            && event->keyval != GDK_KEY_Down) {
                 window->priv->key_events = g_list_prepend (window->priv->key_events,
                                                            gdk_event_copy ((GdkEvent *)event));
         }
@@ -2026,7 +1954,7 @@ maybe_handle_activity (GSWindow *window)
 
         /* if we already have a socket then don't bother */
         if (! window->priv->lock_socket
-            && GTK_WIDGET_IS_SENSITIVE (GTK_WIDGET (window))) {
+            && gtk_widget_get_sensitive (GTK_WIDGET (window))) {
                 g_signal_emit (window, signals [ACTIVITY], 0, &handled);
         }
 
@@ -2130,8 +2058,8 @@ gs_window_real_size_request (GtkWidget      *widget,
         window = GS_WINDOW (widget);
         bin = GTK_BIN (widget);
 
-        if (bin->child && GTK_WIDGET_VISIBLE (bin->child)) {
-                gtk_widget_size_request (bin->child, requisition);
+        if (gtk_bin_get_child (bin) && gtk_widget_get_visible (gtk_bin_get_child (bin))) {
+                gtk_widget_size_request (gtk_bin_get_child (bin), requisition);
         }
 
         old_geometry = window->priv->geometry;
@@ -2141,7 +2069,7 @@ gs_window_real_size_request (GtkWidget      *widget,
         requisition->width  = window->priv->geometry.width;
         requisition->height = window->priv->geometry.height;
 
-        if (! GTK_WIDGET_REALIZED (widget)) {
+        if (!gtk_widget_get_realized (widget)) {
                 return;
         }
 
@@ -2184,6 +2112,14 @@ gs_window_is_obscured (GSWindow *window)
         return window->priv->obscured;
 }
 
+gboolean
+gs_window_is_dialog_up (GSWindow *window)
+{
+        g_return_val_if_fail (GS_IS_WINDOW (window), FALSE);
+
+        return window->priv->dialog_up;
+}
+
 static void
 window_set_obscured (GSWindow *window,
                      gboolean  obscured)
@@ -2260,26 +2196,6 @@ gs_window_class_init (GSWindowClass *klass)
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
-        signals [DIALOG_UP] =
-                g_signal_new ("dialog-up",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GSWindowClass, dialog_up),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
-        signals [DIALOG_DOWN] =
-                g_signal_new ("dialog-down",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (GSWindowClass, dialog_down),
-                              NULL,
-                              NULL,
-                              g_cclosure_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
 
         g_object_class_install_property (object_class,
                                          PROP_OBSCURED,
@@ -2288,6 +2204,13 @@ gs_window_class_init (GSWindowClass *klass)
                                                                NULL,
                                                                FALSE,
                                                                G_PARAM_READABLE));
+        g_object_class_install_property (object_class,
+                                         PROP_DIALOG_UP,
+                                         g_param_spec_boolean ("dialog-up",
+                                                               NULL,
+                                                               NULL,
+                                                               FALSE,
+                                                               G_PARAM_READABLE));
         g_object_class_install_property (object_class,
                                          PROP_LOCK_ENABLED,
                                          g_param_spec_boolean ("lock-enabled",
@@ -2400,6 +2323,7 @@ gs_window_init (GSWindow *window)
 
         window->priv->drawing_area = gtk_drawing_area_new ();
         gtk_widget_show (window->priv->drawing_area);
+        gtk_widget_set_app_paintable (window->priv->drawing_area, TRUE);
         gtk_box_pack_start (GTK_BOX (window->priv->vbox), window->priv->drawing_area, TRUE, TRUE, 0);
         create_info_bar (window);
 
@@ -2451,8 +2375,8 @@ gs_window_finalize (GObject *object)
 
         gs_window_dialog_finish (window);
 
-        if (window->priv->background_pixmap) {
-               g_object_unref (window->priv->background_pixmap);
+        if (window->priv->background_surface) {
+               cairo_surface_destroy (window->priv->background_surface);
         }
 
         G_OBJECT_CLASS (gs_window_parent_class)->finalize (object);
@@ -2470,6 +2394,7 @@ gs_window_new (GdkScreen *screen,
                                "screen", screen,
                                "monitor", monitor,
                                "lock-enabled", lock_enabled,
+                               "app-paintable", TRUE,
                                NULL);
 
         return GS_WINDOW (result);
